function [beta alpha_sv w b pos obj r] = RSVMPlusl1_withoutMu(X, Y, basicKernels, compDist_vect_form, C, C_r, option, largeScale)
%% RSVM+ presented in our paper. 
% RSVMPlusl1_withoutMu - RSVMPlus without an explicit scaling factor Mu. L1 is used for SVM slack variables
% This is solved in dual form, unlike RSVMPlus_QCQP which solves the same problem but in primal form.
% Input:
% X : data, rows are instances, columns are features, size [NxD]
% Y : label, size [Nx1]

% basicKernels: NxNxD matrix of  basic kernel, last dimension is the kernel index
% C         : SVM hyperparameter
% option    : mkl algorithm hyperparameter
%       option.nbitermax : maximal number of iterations (default 1000)
%       option.algo      : selecting algorithm svmclass (default) or svmreg
%       option.seuil     : threshold for zeroing kernel coefficients
%                          (default 1e-12) during the first
%                          option.seuilitermax
%       option.seuilitermax : threshold weigth when the nb of iteration is
%                           lower than this value
%       option.sigmainit  : initial kernel coefficient (default average)
%       option.alphainit : initial Lagrangian coefficient
%       option.lambdareg : ridge regularization added to kernel's diagonal
%                          for SVM (default 1e-10)
%       option.numericalprecision    : force to 0 weigth lower than this
%                                     value (default = eps)
%
% Outputs
% mu            : the weights of kernel.
% alpha_sv      : the weigthed lagrangian of the support vectors: alpha_sv =w =(alpha(supportvector).*ysupport);
% b             : the bias (b)
% pos           : the indices of Support  vectors
%% Copyright@Huyen Do 2012
% Some parts are inherited from SimpleMKL toolbox.

%% Initialize params

[n] = length(Y);

if ~isfield(option,'nbitermax');
    nloopmax = 1000;
else
    nloopmax = option.nbitermax;
end;

if ~isfield(option,'seuil');
    seuil = 1e-12;
else
    seuil = option.seuil;
end
if ~isfield(option,'seuilitermax')
    option.seuilitermax = 20;
end;
if ~isfield(option,'seuildiffsigma');
    option.seuildiffsigma = 1e-5;
end
if ~isfield(option,'seuildiffconstraint');
    option.seuildiffconstraint = 0.05;
end

if ~isfield(option,'lambdareg');
    lambdareg = 1e-10;
    option.lambdareg = 1e-10;
else
    lambdareg = option.lambdareg;
end

if ~isfield(option,'numericalprecision');
    option.numericalprecision = 1e-5;
end;


%------------------------------------------------------
% choosing the stopping criterion
%------------------------------------------------------
option.stopvariation    = 1; % use variation of weights for stopping criterion
option.stopKKT          = 0; % set to 1 if you use KKTcondition for stopping criterion [not yet implemented]
option.stopdualitygap   = 0; % set to 1 for using duality gap for stopping criterion   [not yet implemented]

%------------------------------------------------------
% choosing the stopping criterion value
%------------------------------------------------------
option.seuildiffsigma       = 1e-3;        % stopping criterion for weight variation
option.seuildiffconstraint  = 0.1;         % stopping criterion for KKT
option.seuildualitygap      = 0.01;       % stopping criterion for duality gap
%------------------------------------------------------
% some algorithms paramaters
%------------------------------------------------------
option.firstbasevariable =  'first'; % tie breaking method for choosing the base
% variable in the reduced gradient method
option.nbitermax    = 500;             % maximal number of iteration
option.seuil        = 0;                   % forcing to zero weights lower than this
option.seuilitermax = 10;           % value, for iterations lower than this one

option.miniter      = 0;                 % minimal number of iterations
option.verbosesvm   = 0;              % verbosity of inner svm algorithm
option.efficientkernel      = 1;         % use efficient storage of kernels

verbose = 0;

%---------------------------------------------------------
%Initialize mu

if ~isfield(option,'betaInit');
    numInstances = size(X,1);
    numBeta =  numInstances * (numInstances - 1)/2;
    beta = zeros(numBeta, 1);
    beta(1) = C_r;
    %beta = C_r * ones(numBeta,1)/numBeta;
    %beta = C_r * beta/sum(beta);
    if(numBeta > 1000) 
        option.seuildiffsigma = 0.01; 
    end
       
else 
    beta = option.betaInit ;
end;

%End Initialize mu
% -----------------------------------------------------------

if isfield(option,'alphainit');
    alphainit = option.alphainit;
else
    alphainit  = zeros(length(Y),1);
end;

%--------------------------------------------------------------------------------
% Options used in subroutines
%--------------------------------------------------------------------------------
if ~isfield(option,'goldensearch_deltmax');
    option.goldensearch_deltmax = 1e-1;
end
if ~isfield(option,'goldensearchmax');
    optiongoldensearchmax = 1e-8;
end;
if ~isfield(option,'firstbasevariable');
    option.firstbasevariable = 'first';
end;

%------------------------------------------------------------------------------%
% Initialize
%--------------------------------------------------------------------------

nloop   = 0;
loop    = 1;
status  = 0;
numericalaccuracy        = 1e-9;
goldensearch_deltmaxinit = option.goldensearch_deltmax;


% End Initialize parameters
%% -----------------------------------------------------------


%-----------------------------------------
% Initializing SVM
%------------------------------------------
% kernel matrix with a fixed value of beta
kernel = combinedComponentKernels(basicKernels, beta, compDist_vect_form, largeScale);

% standard SVM with that weighted kernel
[alpha, alpha_sv, obj, w_sqd, b, pos, train_err] = SVM(X, Y, kernel, option.lambdareg, C, alphainit);
% the obj function of SVM should be bigger than 0, otherwise alpha = 0 is
% already an optimal value. 
% the weighted lagrangian of the support vectors: alpha_sv =w =(alpha(sv).*ysupport);

%fprintf('obj at beginning: %f\n', obj);

[grad] = gradRSVMPlusl1_withoutMu(basicKernels,pos,alpha_sv, beta, compDist_vect_form, largeScale);

%% check gradient
% checkGradients(beta, basicKernels, X, Y, lambdareg, C, compDist_vect_form, grad)


%%
betaOld   = beta; 

history.obj = [];
history.beta  = [];
history.KKTconstraint   = [1];
history.dualitygap      = [1];
%------------------------------------------------------------------------------%
% Update Main loop
%------------------------------------------------------------------------------%

while loop & nloopmax >0 ;

    nloop = nloop+1;
    
    history.mu  = [history.beta;beta];
    history.obj = [history.obj obj];

    %-----------------------------------------
    % Update weigths mu
    %-----------------------------------------
    t = cputime ;
    
  
    [beta alpha_sv,b,pos,obj] = RSVMPlusl1_withoutMu_update(basicKernels,beta,pos,alpha_sv,b,C,C_r,Y,grad,-obj, compDist_vect_form, option,largeScale) ;
   
    %-----------------------------------------
    % Thresholding
    %-----------------------------------------

    if seuil ~= 0 & max(beta) > seuil & nloop < option.seuilitermax
        beta = (beta.*(beta > seuil))*sum(beta)/sum(beta .* (beta > seuil));
    end;

    %-------------------------------
    % Numerical cleaning
    %-------------------------------
    %beta(find(abs(beta < option.numericalprecision))) = 0;
    %beta = C_r *  beta / sum(beta);
    
    %-----------------------------------------------------------
    % Enhance accuracy of line search if necessary
    %-----------------------------------------------------------
    if max(abs(beta-betaOld)) < option.numericalprecision & option.goldensearch_deltmax > optiongoldensearchmax
        option.goldensearch_deltmax = option.goldensearch_deltmax/10;
    elseif option.goldensearch_deltmax ~= goldensearch_deltmaxinit
        option.goldensearch_deltmax * 10;
    end;


    %----------------------------------------------------
    % process approximate KKT conditions
    %----------------------------------------------------
%     [grad]  = gradRSVMPlusl1_withoutMu(basicKernels,pos,alpha_sv, beta, compDist_vect_form,largeScale);
%     indpos  = find(beta > option.numericalprecision);
%     indzero = find(abs(beta <= option.numericalprecision));
% 
%     KKTconstraint     = abs ((min(grad(indpos)) - max(grad(indpos)))/min(grad(indpos))) ;
%     KKTconstraintZero =  (min(grad(indzero)) >  max(grad(indpos)) );
% 
%     history.KKTconstraint = [history.KKTconstraint KKTconstraint];

    %----------------------------------------------------
    % process duality gap
    %----------------------------------------------------

%     normek = -grad ; % 0.5*Alpsup'*K(pos,pos,i)*Alpsup;
%     
%     %% duality gap = primal - dual   = \sum alpha_i - 2J(beta) + \lambda/2 r + C \sum xi_i
%     %% where xi_i = max(0, 1- y_i( \sum y_j \alpha_j K(j, i) + b ))
%     %% and r =  max(- grad)
%     kernel_tmp  = combinedComponentKernels(basicKernels, beta, compDist_vect_form,largeScale);
%     xi =  1 - Y .* (kernel_tmp(:, pos) * alpha_sv + b); 
%     xi_max  =  max(xi); 
%     if (xi_max < 0 )
%         xi_max = 0; 
%     end
%     
%     r =  max(-grad); 
%     %[xi_max r obj sum(abs(alpha_sv))]
%     % 'svmclass'
%     dualitygap = (sum(abs(alpha_sv)) - 2 * obj + C_r/2 * r + C * xi_max )/abs(obj);
%     %sumK=sumKbeta(K,mu); if we suppose that duality gap of SVM
%     %dualitygap=(max(normek)-0.5*Alpsup'*sumK(pos,pos)*Alpsup)/obj;
% 
%     history.dualitygap = [history.dualitygap dualitygap];

    %------------------------------------------
    %  verbosity
    %------------------------------------------
    if verbose
        if nloop == 1 || rem(nloop,10)==0
            fprintf('--------------------------------------------------\n');
            fprintf('Iter | Obj.    | DiffBetas | DualGap  | KKT C.   |\n');
            fprintf('--------------------------------------------------\n');
        end;
        fprintf('%d   | %8.4f | %6.4f   | \n',[nloop -obj   max(abs(beta-betaOld))  ]);
    end


    %----------------------------------------------------
    % check variation of mu conditions
    %----------------------------------------------------
    if  option.stopvariation==1 & option.stopKKT== 0 & max(abs(beta - betaOld))<option.seuildiffsigma;
        loop = 0;
        fprintf(1,'variation convergence criteria reached \n');
        history.beta  = [history.beta;beta];
        history.obj = [history.obj obj];
    end;
    %----------------------------------------------------
    % check approximate KKT conditions
    %----------------------------------------------------
%     if  option.stopKKT==1 &  option.stopvariation== 0 & ( KKTconstraint < option.seuildiffconstraint & KKTconstraintZero  )
%         loop = 0;
%         fprintf(1,'KKT convergence criteria reached \n');
%         history.beta  = [history.beta;beta];
%         history.obj = [history.obj obj];
%     end;

    %----------------------------------------------------
    % check KKT and variation of mu conditions
    %----------------------------------------------------
%     if  option.stopKKT == 1 & option.stopvariation== 1 & max(abs(beta - betaOld))<option.seuildiffsigma &  ( KKTconstraint < option.seuildiffconstraint & KKTconstraintZero  )
%         loop = 0;
%         fprintf(1,'variation and KKT convergence criteria reached \n');
% 
%         history.beta  = [history.beta;beta];
%         history.obj   = [history.obj obj];
%     end;

    %----------------------------------------------------
    % check duality gap
    %----------------------------------------------------
%     if  option.stopdualitygap == 1 & dualitygap < option.seuildualitygap
%         loop = 0;
%         fprintf(1,'Duality gap criteria reached \n');
% 
%         history.beta  = [history.beta;beta];
%         history.obj = [history.obj obj];
%     end;

    %----------------------------------------------------
    % check for premature convergence
    %----------------------------------------------------
% 
%     if  max(abs(beta - betaOld))<option.numericalprecision & (length(indpos)==1 | abs(max(beta(indpos))-1) < option.numericalprecision) ...
%             & option.goldensearch_deltmax <=optiongoldensearchmax
%         loop = 0;
%         history.beta  = [history.beta;beta];
%         history.obj = [history.obj obj];
%         status = 1;
%         fprintf(1,'Premature Convergence KKT- Duality Gap Not Satisfied\n');
%     end;

    %-----------------------------------------------------
    % check nbiteration conditions
    %----------------------------------------------------
    if nloop >= nloopmax ,
        loop = 0;
        history.beta  = [history.beta;beta];
        history.obj = [history.obj obj];
        status = 2;
        fprintf(1,'maximum number of iterations reached\n')
    end;
    if nloop < option.miniter & loop==0
        loop = 1;
    end;

    %-----------------------------------------------------
    % Updating Variables
    %----------------------------------------------------

    betaOld  = beta ;

end;

w = zeros(size(X,2),1);
for k = 1: size(X,2)
   w(k) = (alpha_sv)' * X(pos,k) / (1 + beta' * compDist_vect_form(:, k)); 
end 

r = sum(beta' * compDist_vect_form); 
%fprintf('obj at the end - %f \n', obj);
%fprintf ('Sum(beta) =    %2.2f  - -- r =  %2.2f  \n', sum(beta), r); 
