function [res] =  innerCV_RMM(Xtrain, Ytrain, Xtest, Ytest, Ktrain, Ktest,  kfold_inner, vectorC, vectorB, randomSeed)
%% RMM - Relative Margin Machine. 
% This function compute the error of one random split of data. 
%% Params: 
% Input: 
%
% Xtrain: training input X, rows are instances and columns are features. 
% Ytrain: training labels. 
% Xtest:  testing intances
% Ytest:  testing labels.
% Ktrain: kernel of (Xtrain, Xtrain)
% Ktest:  kernel of (Xtest, Xtrain)
% kfold_inner: fold number of inner cross validation to choose the optimal hyperparameters. 
%              We use kfold_inner = 5 in our second set of experiments.
% vectorC: set of C values 
%              We choose the optimal C from set vectorC=[0.1 1 10 100 1000]
%              using inner 5-fold cross validation in our experiments
% vectorB: set of B values. 
%               We choose the optimal B from set vectorB=[0.1 1 10 100 1000]
%               using inner 5-fold cross validation in our experiments
% randomSeed: the random seed to permute the instances to create inner fold
%             for inner cross validation (to choose optimal hyperparams)
% Output:
% res:   a structure, including error of the data split, the value of y
%        prediction, and other values. 
%
%% Copyright@Huyen Do 2012. 


% Initialize
lambdareg = 1e-10;
res.kfold_inner = kfold_inner;
res.vectorC = vectorC; 
res.vectorB = vectorB; 
res.randomSeed = randomSeed; 


rand('twister',randomSeed);
randOrder = randperm(length(Ytrain));

outerTrain = 1:length(Ytrain); 
outerTrain = outerTrain(randOrder);

numInnerTest = floor(length(outerTrain)/kfold_inner);

YPredict_inner_err = zeros(length(outerTrain),length(vectorC), length(vectorB));

%
%% Inner cross validation to choose the optimal hyperparameters
tic;   % time counting

for i = 1: kfold_inner
    fprintf('RMM Inner Cross validation.  insideCV =  %g \n', i);
    %index of inner.test and inner.training samples
    index = [];
    if(i < kfold_inner)
        index = ((i-1)*numInnerTest+1):(i*numInnerTest);
        
    else
        index = ((i-1)*numInnerTest+1):length(outerTrain);
        
    end
    innerTest  = outerTrain(index);
    innerTrain = setdiff(outerTrain, innerTest);
    
    % Repeat for different value of C
    for j = 1:length(vectorC)
        C = vectorC(j);
        for k =  1:length(vectorB)
            B = vectorB(k);
   
            %Begin RMM
            [w,lambda,alpha,delta,deltas] = rmm_light(Ktrain(innerTrain,innerTrain), Ytrain(innerTrain), C, B);
            y_predict = Ktrain(innerTest,innerTrain)   * w  + lambda;
            YPredict_inner_err(innerTest,j,k) = (sign(y_predict) ~= Ytrain(innerTest));
            
            fprintf('C = %g , inner CV error = %f \n',  C,  YPredict_inner_err(innerTest,j,k)); 
        end
    end %End RMM
end; % end inner cross validation

inner_err = mean(YPredict_inner_err);  % mean by column or mean by each C, B

min_inner_err = min(min(inner_err));
indC_B = find(inner_err == min_inner_err);


optC_B = cell(length(indC_B),1);
for k  = 1:length(indC_B)
    i = ceil(indC_B(k)/length(vectorC));
    j = indC_B(k) - (i - 1) * length(vectorC);
    optC_B{k} = [vectorC(j) vectorB(i)];
    
end
C       = optC_B{1}(1);
B       = optC_B{1}(2);

% Now we have optimal C from previous cross validation procedure. Train again with the optimal C
% Then test with instances in outerTest

%Begin RMM 
[w,lambda,alpha,delta,deltas] = rmm_light(Ktrain, Ytrain,C, B);     
y_predict = Ktest  * w  + lambda; 
accuracy = mean(sign(y_predict) == Ytest);

C_opt = C;
B_opt = B; 

%% save result
res.time = toc;  % time counting
classifier.w   = w;
classifier.lambda    = lambda;
classifier.error_test   = 1-accuracy;
classifier.y_predict    = y_predict;


res.classifier       = classifier;
res.Ytest            = Ytest;
res.optimalC         = C_opt;
res.optimalB         = B_opt; 
res.allOptC_B        = optC_B;

res.YPrediction      = sign(y_predict); 
res.averageErr       = sum(res.YPrediction  ~= Ytest)/length(Ytest);

fprintf('C_optimal = %g , error = %f \n',  C_opt,  classifier.error_test )
res.averageErr

