function [res] =  innerCV_RSVMPlusl1_withoutMu(Xtrain, Ytrain, Xtest, Ytest,  kfold_inner, vectorC, vectorC_r, randomSeed)
%% RSVMPlusl1_withoutMu (RSVM+ as named in the paper)
% This function compute the error of one random split of data.
% For large scale problem, use function innerCV_RSVMPlusl1_withoutMuLS()
%% Params: 
% Input: 
%
% Xtrain: training input X, rows are instances and columns are features. 
% Ytrain: training labels. 
% Xtest:  testing intances
% Ytest:  testing labels.
% Ktrain: kernel of (Xtrain, Xtrain)
% Ktest:  kernel of (Xtest, Xtrain)
% kfold_inner: fold number of inner cross validation to choose the optimal hyperparameters. 
%              We use kfold_inner = 5 in our second set of experiments.
% vectorC: set of C values 
%              We choose the optimal C from set vectorC=[0.1 1 10 100 1000]
%              using inner 5-fold cross validation in our experiments
% vectorC_r: set of lambda values. 
%               We choose the optimal C_r from set vectorC_r=[0.1 1 10 100 1000]
%               using inner 5-fold cross validation in our experiments
% randomSeed: the random seed to permute the instances to create inner fold
%             for inner cross validation (to choose optimal hyperparams)
% Output:
% res:   a structure, including error of the data split, the value of y
%        prediction, and other values. 
%
%% Copyright@Huyen Do 2012. 

% Initialize
lambdareg = 1e-10;
res.kfold_inner = kfold_inner;
res.lambreg  = lambdareg; 
res.vectorC = vectorC; 
res.vectorC_r = vectorC_r; 
res.randomSeed = randomSeed;

option.lambdareg = lambdareg; 
option.verbose = 1; 

rand('twister',randomSeed);
randOrder = randperm(length(Ytrain));

numFeatures = size(Xtrain,2);


fprintf('Create kernel of size %gx%gx%g \n',length(Ytrain),length(Ytrain), numFeatures);

Ktrain = zeros(length(Ytrain), length(Ytrain), numFeatures);
for k = 1:numFeatures
    Ktrain(:,:,k) = Xtrain(:, k) * Xtrain(:, k)';
end

outerTrain = 1:length(Ytrain); 
outerTrain = outerTrain(randOrder);

numInnerTest = floor(length(outerTrain)/kfold_inner);

YPredict_inner_err= zeros(length(outerTrain),length(vectorC), length(vectorC_r));

%
%% Inner cross validation to choose the optimal hyperparameters
tic; 

for i = 1: kfold_inner
    fprintf('RSVMPlusWithoutMu Cross validation.  insideCV =  %g \n', i);
    index = [];
    if(i < kfold_inner)
        index = ((i-1)*numInnerTest+1):(i*numInnerTest);
        
    else
        index = ((i-1)*numInnerTest+1):length(outerTrain);
        
    end
    innerTest  = outerTrain(index);
    innerTrain = setdiff(outerTrain, innerTest);
    
    % Compute component squared distances
    [compDistance_vect_form]=  componentDistances(Xtrain(innerTrain,:));
    
    % Repeat for different value of C
    for j =  1:length(vectorC)
        C = vectorC(j);
        % Repeat for different value of C_r
        for k = 1:length(vectorC_r)
            C_r = vectorC_r(k);
            
            % RSVMPlus_withoutMu  ==============================         
            [beta  alpha_sv w  b pos obj r ] = RSVMPlusl1_withoutMu(Xtrain(innerTrain,:), Ytrain(innerTrain),Ktrain(innerTrain,innerTrain, :), compDistance_vect_form, C,C_r, option, 0);
                      
            y_predict   = Xtrain(innerTest,:)  * w  + b;
            accuracy    = mean(sign(y_predict) == Ytrain(innerTest));
            
            YPredict_inner_err(innerTest,j,k) = (sign(y_predict) ~= Ytrain(innerTest));    
           
            % end RSVMPlus_withoutMu =============================
        end
    end
end; % end inside cross validation
 
inner_err = mean(YPredict_inner_err); % mean error for each C and C_r

min_inner_err = min(min(inner_err));
indC_Cr = find(inner_err == min_inner_err);

optC_Cr = cell(length(indC_Cr),1);
for k  = 1:length(indC_Cr)
    i = ceil(indC_Cr(k)/length(vectorC));
    j = indC_Cr(k) - (i - 1) * length(vectorC);
    optC_Cr{k} = [vectorC(j) vectorC_r(i)];
    
end
    
% Compute component squared distances
[compDistance_vect_form2]=  componentDistances(Xtrain);

% Now compute with all optimal pair C and C_r. 
% RSVMPlus_constrainedmu==============================

C       = optC_Cr{1}(1);
C_r     = optC_Cr{1}(2);

[beta  alpha_sv w  b pos obj r ] = RSVMPlusl1_withoutMu(Xtrain, Ytrain, Ktrain, compDistance_vect_form2, C, C_r, option, 0);
y_predict = Xtest * w  + b;
accuracy  = mean(sign(y_predict) == Ytest);

C_opt = C;
C_r_opt = C_r;
for i = 2:length(optC_Cr)
    
    C       = optC_Cr{i}(1);
    C_r     = optC_Cr{i}(2);
    [beta_tmp  alpha_sv_tmp w_tmp  b_tmp pos_tmp obj_tmp r_tmp ] = RSVMPlusl1_withoutMu(Xtrain, Ytrain, Ktrain, compDistance_vect_form2, C, C_r, option, 0);
    y_predict_tmp = Xtest  * w_tmp  + b_tmp;
    accuracy_tmp  = mean(sign(y_predict_tmp) == Ytest);
    if(accuracy_tmp > accuracy)
        accuracy    = accuracy_tmp;
        w           = w_tmp;
        b           = b_tmp;
        obj         = obj_tmp;
        r           = r_tmp;
        y_predict   = y_predict_tmp;
        C_opt       = C;
        C_r_opt     = C_r;
    end
       
    
end

fprintf('RSVMPlus_withoutMu Cross validation. optimalC = %g, optC_r = %g, error %f\n', C_opt, C_r_opt, 1-accuracy );

%End RSVMPlus_withoutMu
res.time = toc; 

classifier.error_test =   1 -accuracy;
classifier.y_predict  =  y_predict;
classifier.radius     = r;
classifier.w          = w;
classifier.b          = b;
classifier.obj       = obj; 


res.kernelTooBig     = kernelTooBig;
res.kernelTooTooBig  = kernelTooTooBig;

res.classifier       = classifier;
res.Ytest            = Ytest;
res.YPrediction      = sign(y_predict); 
res.optimalC     = C_opt;
res.optimalCr    = C_r_opt;
res.allOptC_Cr   = optC_Cr;

res.averageErr       =  sum(res.YPrediction  ~= Ytest)/length(Ytest);

fprintf('C_optimal = %g , error = %f \n',  C_opt,  classifier.error_test )
res.averageErr

