function [result] = launch(dataname, data, algorithm, kernelType, splitnum)
%% runBySplit: data are splited into training and testing set
%% Params
% Input: 
% dataname: data name. In the benchmarks dataset we used, dataname can be
%          'thyroid', 'titanic', 'breast_cancer', 'heart'
% data: is formatted as follows:
%   data.x  % all data
%   data.t  % all y label
%   data.train(i,:) % index of training data of split number i
%   data.test(i, :) % index of testing data of split number i
% algorithm: can be 'RMM', 'RSVMPlusl1WithoutMu', 'RSVMPlusl1', 'SVMl1'
%
% kernelType: kernel, can be 'linear', 'poly2', 'poly3', 'gauss1'
% splitnum: since in data, we already have default 100 splits. The splitnum
%           indicates which data split to use. 
%
% Output: 
% result: error of algorithm @algorithm on split @splitnum, on dataset @dataname

%% Copyright@Huyen Do 2012

%% Load data
% standadize the data % always do this!
data.x = normalizemeanstd(data.x);

% training and testing sample of split @splitnum
index.train = data.train(splitnum, :);
index.test   = data.test(splitnum, :);

X.train = data.x(index.train, :); 
X.test = data.x(index.test, :); 

Y.train = data.t(index.train, :); 
Y.test  = data.t(index.test, :); 

%% Initialize set of hyperparameters. 
% In our experiment, we choose C from [ 0.1 1 10 100 1000], and B of RMM, lambda of RSVM+ and RSVM^+_mu from  [ 0.1 1 10 100 1000]

vectorC = [ 0.1 1 10 100 1000];
vectorC_r = [ 0.1 1 10 100 1000];

load('randomSeed.mat'); % the random Seed to create inner folds for inner cross validation. 
% We use the same setting in RMM paper (Shivaswamy&Jebara 2010)
% We use inner 5-fold cross validation to choose the optimal hyperparam. 
kfold_inner = 5 ;       


% Create kernel K
if (nargin > 2)
    switch kernelType
        case 'linear' % linear kernel
            K     = data.x* data.x';
        case 'poly2' % polinimal degree 2
            K     =  kernelmatrix(data.x,3 ,2);
        case 'poly3'
            K     =  kernelmatrix(data.x,3,3);
        case 'gauss1' % Gaussian kernel
            K     =  kernelmatrix(data.x,2,1);
           
    end
end

% Normalize the kernel 
K    = normalizeKernel(K);

kernel.train     = K(index.train, index.train);
kernel.test      = K(index.test, index.train);

switch algorithm
             
    case 'SVMl1'  % Standard SVM with l1 norm of slack variable
        
        [result]= innerCV_SVMl1(X.train, Y.train, X.test, Y.test, kernel.train, kernel.test, kfold_inner, vectorC, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.SVMl1.split' int2str(splitnum) '.result.mat'], 'result' ); 
  
    case 'RMM'  % Relative Margin Machine
        [result]= innerCV_RMM(X.train, Y.train, X.test, Y.test, kernel.train, kernel.test, kfold_inner, vectorC, vectorC_r, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.RMM.split' int2str(splitnum) '.result.mat'], 'result' ); 
    
    case 'RSVMPlusl1WithoutMu'  % RSVMPlusl1WithoutMu
        [result]= innerCV_RSVMPlusl1_withoutMu(X.train, Y.train, X.test, Y.test, kfold_inner,vectorC, vectorC_r, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.RSVMPlusl1WithoutMu.lambreg1e-10.split' int2str(splitnum) '.result.mat'], 'result' ); 
        
    case 'ProximitySpace_RSVMPlusl1WithoutMu'  % ProximitySpace_RSVMPlusl1WithoutMu
        % Compute the proximity space
        X.train = kernel.train;
        X.test  = kernel.test;
        clear K kernel.train kernel.test data
        
        [result]= innerCV_RSVMPlusl1_withoutMu(X.train, Y.train, X.test, Y.test, kfold_inner,vectorC, vectorC_r, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.ProximitySpace_RSVMPlusl1WithoutMu.split' int2str(splitnum) '.result.mat'], 'result' ); 
 
    case 'RSVMPlusl1'  % RSVMPlusl1 % with scaling factor mu
        [result]= innerCV_RSVMPlusl1(X.train, Y.train, X.test, Y.test, kfold_inner,vectorC, vectorC_r, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.RSVMPlusl1.split' int2str(splitnum) '.result.mat'], 'result' ); 
        
 
    case 'ProximitySpace_RSVMPlusl1'  % ProximitySpace_RSVMPlusl1
        % Compute the proximity space
        X.train = kernel.train;
        X.test  = kernel.test;
        clear K kernel.train kernel.test data
        
        [result]= innerCV_RSVMPlusl1(X.train, Y.train, X.test, Y.test, kfold_inner,vectorC, vectorC_r, randomSeed);
        result.splitnum = splitnum;
        save([ dataname '.' kernelType '.ProximitySpace_RSVMPlusl1.split' int2str(splitnum) '.result.mat'], 'result' ); 
 
        
end


    